<?php

/**
 * GuestBook extension for Mediawiki.
 *
 *
 * Copyright (C) 2005-2007 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * @package MediaWiki
 * @subpackage Extensions
 * @author Tino Reichardt <milky dash gbook at mcmilk dot de>
 */

if (!defined('MEDIAWIKI')) die();

/* if set to false, no captcha is needed! */
$want_captcha = true;

/* sysop's can moderate the guestbook */
$wgGroupPermissions['sysop']['gbook'] = true;
$myExtensionPath = "$wgScriptPath/extensions/gbook";

/* package definitions */
$wgExtensionFunctions[] = "wfGuestbookSpecialpage";
$wgExtensionCredits['specialpage'][] = array(
	'author' => 'Tino Reichardt',
	'name' => 'Guestbook',
	'version' => '0.11',
	'url' => 'http://www.mcmilk.de/wiki/Category:Wiki-Guestbook'
);

/* disable captcha, if needed functions don't exist! */
if (!function_exists("imagecreatetruecolor")) { $want_captcha = false; }
if (!function_exists("imagettftext")) { $want_captcha = false; }

function wfGuestbookSpecialpage() {
  global $IP, $wgMessageCache;
  require_once( "$IP/includes/SpecialPage.php" );

  /* here we define all routines for our guestbook */
  class Guestbook extends SpecialPage {
    var $gbName, $gbOrigin, $gbEmail, $gbHomepage, $gbEntry, $gbCaptcha, $gbIP,
        $gbBrowser, $gbReplyto;
    var $mySpecialPage;

    function Guestbook() {
      SpecialPage::SpecialPage( 'Guestbook' );
      $this->includable(true);
      $this->listed(false);
    }

    function execute() {
      global $wgOut, $wgRequest, $wgUser, $myExtensionPath;

      $sp = Title::newFromText("Special:Guestbook");
      $this->mySpecialPage = $sp->getLocalURL();

      $wgOut->setPagetitle($this->text("headline"));
      $wgOut->setArticleRelated(false); /* removes the getID "Fatal error" ;) */
      $wgOut->addLink(array(
	'rel'	=> 'stylesheet',
	'type'	=> 'text/css',
	'media'	=> 'screen,projection',
	'href'	=> "$myExtensionPath/gbook.css"
      ));

      /* add some new entry */
      $this->checktable();

      if ($wgRequest->getText( 'action' ) == "add") {
        if (!$this->ParseUserInput()) {
          $this->ShowSubmitForm(0);
	  return;
	}
      }

      /* reply to some entry */
      if (is_numeric($wgRequest->getText('reply'))) {
        if (!$this->ParseUserInput()) {
          $this->ShowSubmitForm($wgRequest->getText( 'reply' ));
	  return;
	}
      }

      /* moderate some entry */
      if ($wgUser->isAllowed('gbook')) {
        if ($wgRequest->getText('mod') != "") {
          if (!$this->ParseModInput()) {
            $this->ShowModForm();
            return;
          }
        }
      }

      $p=$wgRequest->getText( 'p' );
      if (is_numeric($p)) {
        $this->ShowEntryPage($p);
      } else {
        $this->ShowEntryPage(0);
      }

      return;
    }

    function success($msg) {
      global $wgOut;

      $wgOut->addHTML('<div class="successbox">');
      $wgOut->addHTML($msg);
      $wgOut->addHTML('</div><div class="visualClear"></div>');

      return;
    }

    function error($msg) {
      global $wgOut;

      $wgOut->addHTML('<div class="errorbox">');
      $wgOut->addHTML($msg);
      $wgOut->addHTML('</div><div class="visualClear"></div>');

      return;
    }

    function checktable() {
      /* get the old entry stuff from the database */
      $dbr =& wfGetDB( DB_MASTER );
      $gb = $dbr->tableName('guestbook');

      $sql="
      CREATE TABLE IF NOT EXISTS $gb (
        gb_id INT AUTO_INCREMENT,
        gb_replyto INT,
        gb_timestamp CHAR(14) BINARY,
        gb_name BLOB,
        gb_origin BLOB,
        gb_email BLOB,
        gb_homepage BLOB,
        gb_entry BLOB,
        gb_ip BLOB,
        gb_browser BLOB,
        PRIMARY KEY (gb_id)
      );";
      $dbr->query($sql, 'wfGuestbookSpecial');
      $res = $dbr->query('SHOW COLUMNS FROM ' . $gb, 'wfGuestbookSpecial');
      $obj = $dbr->fetchObject($res);
      $gb_replyto_exists = false; /* new since 0.8 */
      while ($obj = $dbr->fetchObject( $res )) {
        foreach($obj as $column) {
          if ($column == 'gb_replyto') {
            $gb_replyto_exists = true;
          }
        }
      }
      $dbr->freeResult($res);

      if (!$gb_replyto_exists) {
        $sql = "ALTER TABLE $gb ADD gb_replyto INT";
        $dbr->query($sql, 'wfGuestbookSpecial');
      }

      return;
    }

    function text($str) {
      global $wgUser, $IP;

      /* we may add some other languages here as well */
      $lang = $wgUser->getOption('language');
      if (is_file("$IP/extensions/gbook/gbook.i18n-$lang.php")) {
        require("gbook.i18n-$lang.php");
      } else {
        require("gbook.i18n-en.php");
      }
      if (!isset($msg[$str])) { return "error @ text($str)"; }

      return $msg[$str];
    }

    /**
     * get user input (submitting new entries)
     */
    function ParseUserInput() {
      require_once("captcha-class.php");
      global $wgRequest, $want_captcha;

      /* we are called the first time */
      $this->gbReplyto = $wgRequest->getText( 'gbReplyto' );
      if ($this->gbReplyto == 0) {
        $this->gbReplyto = NULL;
      }
      $this->gbSubmit = $wgRequest->getText( 'gbSubmit' );
      if ($this->gbSubmit == "") return;

      /* required */
      $this->gbName = $wgRequest->getText( 'gbName' );
      $this->gbEntry = $wgRequest->getText( 'gbEntry' );
      $this->gbCaptcha = $wgRequest->getText( 'gbCaptcha' );

      /* optional */
      $this->gbOrigin = $wgRequest->getText( 'gbOrigin' );
      $this->gbEmail = $wgRequest->getText( 'gbEmail' );
      $this->gbHomepage = $wgRequest->getText( 'gbHomepage' );

      if ($this->gbName == "") {
        $this->error($this->text("err_name"));
	return;
      }
      if ($this->gbEntry == "") {
        $this->error($this->text("err_entry"));
	return;
      }

      /* captcha on/off */
      if ($want_captcha) {
        if ($this->gbCaptcha == "") {
          $this->error($this->text("err_captcha"));
	  return;
        }

        $cap = new captcha();
        if ($cap->verify($this->gbCaptcha) != true) {
  	  $this->error($this->text("captcha_fail"));
  	  return;
        }
        unset($cap);
      }

      $gbTimestamp = time(0);
      $dbw =& wfGetDB( DB_MASTER );
      $dbw->insert('guestbook', array(
        'gb_timestamp'  => $gbTimestamp,
	'gb_name' => $this->gbName,
	'gb_entry' => $this->gbEntry,
	'gb_email' => $this->gbEmail,
	'gb_origin' => $this->gbOrigin,
	'gb_homepage' => $this->gbHomepage,
	'gb_replyto' => $this->gbReplyto,
	'gb_ip' => $_SERVER['REMOTE_ADDR'],
	'gb_browser' => $_SERVER['HTTP_USER_AGENT']),
	'Guestbook::ParseUserInput', array('IGNORE'));

      $this->success($this->text("entry_okay"));
      return 1;
    }

    /**
     * get moderator input (editing/deleting old entries)
     */
    function ParseModInput() {
      require_once("captcha-class.php");
      global $wgRequest;

      $id = $wgRequest->getText('mod');
      if (!is_numeric($id)) {
        $this->error($this->text("err_noentry"));
      }

      /* get the old entry stuff from the database */
      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT * FROM $gb WHERE gb_id=$id";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      $obj = $dbr->fetchObject($res);
      $dbr->freeResult($res);

      $this->gbName     = $obj->gb_name;
      $this->gbEntry    = $obj->gb_entry;
      $this->gbEmail    = $obj->gb_email;
      $this->gbOrigin   = $obj->gb_origin;
      $this->gbHomepage = $obj->gb_homepage;
      $this->gbIP       = $obj->gb_ip;
      $this->gbBrowser  = $obj->gb_browser;

      /* when called the first time, do nothing special */
      $this->gbSubmit = $wgRequest->getText( 'gbSubmit' );
      if ($this->gbSubmit == "") {
        return 0;
      }

      $todo = $wgRequest->getText( 'todo' );
      switch ($todo) {
      case $this->text("b_change"):
        $dbw =& wfGetDB( DB_MASTER );
        $dbw->update('guestbook',
	 array(
	  'gb_name'       => $wgRequest->getText('gbName'),
	  'gb_entry'      => $wgRequest->getText('gbEntry'),
	  'gb_email'      => $wgRequest->getText('gbEmail'),
	  'gb_origin'     => $wgRequest->getText('gbOrigin'),
	  'gb_homepage'   => $wgRequest->getText('gbHomepage'),
	  'gb_ip'         => $wgRequest->getText('gbIP'),
	  'gb_browser'    => $wgRequest->getText('gbBrowser')),
	 array('gb_id' => "$id"),
	  'Guestbook::ParseModInput', array('IGNORE'));
	$this->success($this->text("change_okay"));
	return 1;
      case $this->text("b_delete"):
	$sql="DELETE FROM $gb WHERE gb_id=$id";
	$res = $dbr->query($sql, 'wfGuestbookSpecial');
	$this->success($this->text("delete_okay"));
	return 1;
      }

      return 0;
    }

    function EntryExist($id) {
      global $wgOut, $wgUser, $myExtensionPath;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql = "SELECT gb_id FROM $gb WHERE gb_id=$id";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      $obj = $dbr->fetchObject($res);
      $dbr->freeResult($res);

      return is_numeric($obj->gb_id);
    }

    function ShowEntry($id) {
      global $wgOut, $wgUser, $myExtensionPath;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT * FROM $gb WHERE gb_id=$id";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      while ($obj = $dbr->fetchObject( $res )) {
	$name=$obj->gb_name;
        $ts=date("Y-m-d @ H:m:s", $obj->gb_timestamp);
	$origin=$obj->gb_origin;
	$entry=str_replace("\n", "<br />", $obj->gb_entry);
	$browser=$obj->gb_browser;
	$ip=$obj->gb_ip;
	$id=$obj->gb_id;
	$replyto=$obj->gb_replyto;
	$b_reply=$this->text('b_reply');

	/* create to homepage link */
	$hp = $obj->gb_homepage;
	if ($hp != "") {
	  if (strstr($hp,'://')) {
	    $url = $hp;
	  } else {
	    $url = 'http://' . $hp;
	  }
	  $hp_link="(<a href=\"$url\">$hp</a>)";
	} else {
	  $hp_link="";
	}

	/* create email link */
	$email=$obj->gb_email;
	if ($email != "") {
	  $email=str_replace("@", "(at)", $email);
	  $email=str_replace(".", "(dot)", $email);
	  $email=str_replace("-", "(dash)", $email);
	  $email_link="<a href=\"mailto:$email\"></a>";
	} else {
	  $email_link="";
	}

        $moderate  = "";
        if ($wgUser->isAllowed('gbook')) {
          $moderate .= "<a href=\"" . "$this->mySpecialPage" . "?mod=$id\">";
	  $moderate .= "<img alt=\"modify\" src=\"$myExtensionPath/gbook_tool.png\" />";
	  $moderate .= "</a>";
        }
        $moderate .= "<a href=\"" . "$this->mySpecialPage" . "?reply=$id\">";
	$moderate .= "<img alt=\"$b_reply\" src=\"$myExtensionPath/gbook_reply.png\" />";
	$moderate .= "</a>";

        $out = <<<ENDFORM
<table width="100%" cellspacing="0" cellpadding="0">
<tr>
 <td width="100%"><b>$name</b> $email_link $hp_link</td>
 <td nowrap><small>$ts</small></td>
</tr><tr>
 <td colspan="2">
ENDFORM;
        $wgOut->addHTML($out);
        $wgOut->addWikiText($entry);
        $out = <<<ENDFORM
</td>
</tr>
</table>

<table width="100%" cellspacing="0" cellpadding="0"><tr>
 <td nowrap align="left">$moderate</td>
 <td align="right" width="100%" valign="bottom"><small>[$ip / $browser]</small></td>
</tr>
</table>
ENDFORM;
        $wgOut->addHTML($out);
        $this->ShowEntryReplies($id);
      }
      $dbr->freeResult( $res );
    }

    function ShowEntryReplies($id) {
      global $wgOut, $wgUser, $myExtensionPath;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT gb_id FROM $gb WHERE gb_replyto=$id";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      while ($obj = $dbr->fetchObject( $res )) {
        $wgOut->addHTML( '<div class="gbook">' );
	$this->ShowEntry($obj->gb_id);
        $wgOut->addHTML( '</div>' );
      }
    }

    function ShowEntryPage($page) {
      global $wgOut, $wgUser, $myExtensionPath;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT COUNT(*) AS total FROM $gb WHERE gb_replyto is NULL";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      $obj = $dbr->fetchObject( $res );
      $dbr->freeResult( $res );
      $total = $obj->total;

      $limit=$wgUser->getOption('rows'); /* entries per page */
      $start=$limit*$page;
      if ($start > $total) $start=$total-$limit;
      for ($pages=0; $limit*$pages<$total; $pages++);

      $wgOut->addHTML( '<div class="gbook">' );
      $wgOut->addWikiText($this->text("new_entry"));
      if ($pages > 1) {
        $wgOut->addWikiText("----");
        $wgOut->addHTML("<b>");
        for ($p=$pages; $limit*$p>=0; $p--) {
          $x = $pages - $p;
          if ($x == $page) {
            $wgOut->addHTML("$p ");
	  } else {
            $wgOut->addHTML("<a href=\"$this->mySpecialPage?p=$x\">$p</a> ");
	  }
        }
        $wgOut->addHTML("</b>");
      }
      $wgOut->addHTML( '</div>' );

      $sql="SELECT gb_id FROM $gb WHERE gb_replyto is NULL
        ORDER BY gb_timestamp DESC LIMIT $start,$limit";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      while ($obj = $dbr->fetchObject( $res )) {
        $wgOut->addHTML( '<div class="gbook">' );
	$this->ShowEntry($obj->gb_id);
        $wgOut->addHTML( '</div>' );
      }

      if ($pages > 1) {
        $wgOut->addHTML( '<div class="gbook">' );
        $wgOut->addHTML("<b>");
        for ($p=$pages; $limit*$p>=0; $p--) {
          $x = $pages - $p;
          if ($x == $page) {
            $wgOut->addHTML("$p ");
	  } else {
            $wgOut->addHTML("<a href=\"$this->mySpecialPage?p=$x\">$p</a> ");
	  }
        }
        $wgOut->addHTML("</b>");
        $wgOut->addHTML( '</div>' );
      }

      return;
    }

    /**
     * show the submitting form
     */
    function ShowSubmitForm($replyto) {
      global $wgOut, $wgUser, $myExtensionPath, $want_captcha;

      $gb1=$this->text("f_name");
      $gb2=$this->text("f_origin");
      $gb3=$this->text("f_email");
      $gb4=$this->text("f_homepage");
      $gb5=$this->text("f_entry");
      $gb6=$this->text("f_captcha");
      $gb7=$this->text("show_entries");
      $lnk="$this->mySpecialPage";
      $cap="$myExtensionPath/captcha.php";
      $alt=$this->text("captcha_desc");
      $b_reset=$this->text("b_reset");
      $b_submit=$this->text("b_submit");
      $hl=$this->text("hl_create");
      $lng=$wgUser->getOption('language');

      if ($replyto != 0) {
        if (!$this->EntryExist($replyto)) {
          $this->error($this->text("err_noentry"));
          unset($replyto);
        } else {
          $b_submit=$this->text("b_reply");
          $wgOut->addHTML( '<div class="gbook">' );
          $this->ShowEntry($replyto);
          $wgOut->addHTML( '</div>' );
        }
      }

      $out=<<<ENDFORM
<div id="gbook">
<h2>$hl</h2>
<form name="guestbook" method="post" action="">
<input type="hidden" name="gbSubmit" value="okay" />
<input type="hidden" name="gbReplyto" value="$replyto" />
<table width="100%" border="0">
<tr>
 <td valign="top">$gb1:</td>
 <td><input tabindex="1" type="text" name="gbName" size="40" value="$this->gbName" /></td>
ENDFORM;
 if ($want_captcha) {
   $out.=<<<ENDFORM
 <td valign="top" rowspan="4" align="right">
 <p><a href="http://$lng.wikipedia.org/wiki/Captcha" title="$alt">$gb6</a>: <input tabindex="6" type="text" name="gbCaptcha" size="8" value="" />
 <p><img alt="$alt" src="$cap"></img>
 </td>
ENDFORM;
 } else {
   $out.=<<<ENDFORM
   <td rowspan="4"></td>
ENDFORM;
 }
      $out.=<<<ENDFORM
</tr><tr>
 <td valign="top">$gb2:</td>
 <td><input tabindex="2" type="text" name="gbOrigin" size="40" value="$this->gbOrigin" /></td>
</tr><tr>
 <td valign="top">$gb3:</td>
 <td><input tabindex="3" type="text" name="gbEmail" size="40" value="$this->gbEmail" /></td>
</tr><tr>
 <td valign="top">$gb4:</td>
 <td><input tabindex="4" type="text" name="gbHomepage" size="40" value="$this->gbHomepage" /></td>
</tr><tr>
 <td valign="top">$gb5:</td>
 <td colspan="2"><textarea tabindex="5" name="gbEntry" cols="40" rows="7">$this->gbEntry</textarea></td>
</tr><tr>
 <td colspan="2"><a href="$lnk">$gb7</a></td>
 <td align="right">
  <input tabindex="7" type="submit" value="$b_submit"/>
  <input tabindex="8" type="reset" value="$b_reset"/>
 </td>
</tr>
</table>
</form>
</div>
ENDFORM;
      $wgOut->addHTML( $out );

      return;
    }

    /**
     * show the moderation form
     */
    function ShowModForm() {
      global $wgOut, $wgUser;

      $gb1=$this->text("f_name");
      $gb2=$this->text("f_origin");
      $gb3=$this->text("f_email");
      $gb4=$this->text("f_homepage");
      $gb5=$this->text("f_entry");
      $gb6=$this->text("f_ip");
      $gb7=$this->text("f_browser");
      $gb8=$this->text("show_entries");
      $lnk="$this->mySpecialPage";
      $b_change=$this->text("b_change");
      $b_delete=$this->text("b_delete");
      $b_reset=$this->text("b_reset");
      $hl=$this->text("hl_modify");
      $lng=$wgUser->getOption('language');

      $out=<<<ENDFORM
<div id="gbook">
<h2>$hl</h2>
<form name="guestbook" method="post" action="">
<input type="hidden" name="gbSubmit" value="okay" />
<input type="hidden" name="gbReplyto" value="$this->gbReplyto" />
<table width="100%" border="0">
<tr>
 <td valign="top">$gb1:</td>
 <td><input tabindex="1" type="text" name="gbName" size="40" value="$this->gbName" /></td>
</tr><tr>
 <td valign="top">$gb2:</td>
 <td><input tabindex="2" type="text" name="gbOrigin" size="40" value="$this->gbOrigin" /></td>
</tr><tr>
 <td valign="top">$gb3:</td>
 <td><input tabindex="3" type="text" name="gbEmail" size="40" value="$this->gbEmail" /></td>
</tr><tr>
 <td valign="top">$gb4:</td>
 <td><input tabindex="4" type="text" name="gbHomepage" size="40" value="$this->gbHomepage" /></td>
</tr><tr>
 <td valign="top">$gb5:</td>
 <td colspan="2"><textarea tabindex="5" name="gbEntry" cols="40" rows="7">$this->gbEntry</textarea></td>
</tr><tr>
 <td valign="top">$gb6:</td>
 <td><input tabindex="6" type="text" name="gbIP" size="40" value="$this->gbIP" /></td>
</tr><tr>
 <td valign="top">$gb7:</td>
 <td><input tabindex="7" type="text" name="gbBrowser" size="40" value="$this->gbBrowser" /></td>
</tr><tr>
 <td colspan="2"><a href="$lnk">$gb8</a></td>
 <td align="right" nowrap>
  <input name="todo" tabindex="8" type="submit" value="$b_change"/>
  <input tabindex="9" type="reset" value="$b_reset"/> |
  <input name="todo" tabindex="10" type="submit" value="$b_delete"/>
 </td>
</tr>
</table>
</form>
</div>
ENDFORM;
      $wgOut->addHTML( $out );

      return;
    }
  } /* class */

  SpecialPage::addPage( new Guestbook );
}

?>
